@@ -11,11 +11,14 @@ from bits.github import GitHub
 class Update(object):
     """Update class."""
 
-    def __init__(self, auth, github):
+    def __init__(self, auth=None, github=None):
         """Initialize a class instance."""
         self.auth = auth
         self.github = github
-        self.verbose = auth.verbose
+
+        self.verbose = False
+        if self.github:
+            self.verbose = self.github.verbose
 
         # data
         self.github_ids = {}
@@ -28,8 +31,8 @@ class Update(object):
     def _get_firestore_github_users(self):
         """Return a dict of cached GitHub users from Firestore."""
         github_users = {}
-        firestore = self.github.firestore(self.auth)
-        for doc in firestore.db.collection('github_users').stream():
+        firestore = self.github.firestore()
+        for doc in firestore.app.collection('github_users').stream():
             github_id = doc.id
             github_users[github_id] = doc.to_dict()
         return github_users
@@ -91,7 +94,7 @@ class Update(object):
 
     def _get_people(self):
         """Return a dict of people."""
-        firestore = self.github.firestore(self.auth)
+        firestore = self.github.firestore()
         self.people = firestore.get_people_dict(key='google_id')
         return self.people
 
@@ -117,7 +120,7 @@ class Update(object):
 
     def _get_users(self):
         """Return a dict of users."""
-        datastore = self.github.datastore(self.auth)
+        datastore = self.github.datastore()
         self.users = datastore.get_users_dict()
 
         # get users by github_id and google_id
@@ -227,6 +230,9 @@ class Update(object):
 
         return team_members
 
+    #
+    # Update Team Members
+    #
     def update_members(self):
         """Update members in GitHub."""
         current = self._get_members()
@@ -250,23 +256,26 @@ class Update(object):
             print('\nMembers to invite: {}'.format(len(invite)))
             for login in sorted(invite):
                 print('   + {}'.format(login))
-                # self.github.invite_org_member(login)
+                self.github.invite_org_member(login)
 
         if remove:
             print('\nMembers to remove: {}'.format(len(remove)))
             for login in sorted(remove):
                 print('   - {}'.format(login))
-                # self.github.remove_org_member(login)
+                self.github.remove_org_member(login)
 
         if uninvite:
             print('\nInvitations to cancel: {}'.format(len(uninvite)))
             for login in sorted(uninvite):
                 print('   - {}'.format(login))
-                # self.github.remove_org_member(login)
+                self.github.remove_org_member(login)
 
         if self.verbose:
             print('Done updating GitHub members.')
 
+    #
+    # Update Teams
+    #
     def update_team(self, slug, team_id, email):
         """Update a single team."""
         # get current team members
@@ -341,7 +350,7 @@ class Update(object):
         print('Found {} People in BITSdb.'.format(len(self.people)))
 
         # firestore - get team syncs
-        firestore = self.github.firestore(self.auth)
+        firestore = self.github.firestore()
         team_syncs = firestore.get_team_syncs_dict()
         print('Found {} GitHub Team Syncs in BITSdb.'.format(len(team_syncs)))
 
@@ -354,18 +363,70 @@ class Update(object):
         if self.verbose:
             print('Done updating GitHub teams.')
 
-    def update_user(self, user):
-        """Update a single user."""
-        # check for current record
-        github_id = user['github_id']
-        login = user['login'].lower()
-        token = user['token']
+    #
+    # Update Team Syncs
+    #
+    def update_team_syncs(self):
+        """Update Team Syncs from On-Prem Mongo to Cloud Firestore."""
+        g = self.auth.google()
+        g.auth_service_account(g.scopes, g.subject)
 
+        # get team syncs from source of truth - bitsdb mongo
+        m = self.auth.mongo()
+        mongo_team_syncs = m.get_collection('github_team_sync')
+        print('Found {} GitHub Team Syncs in BITSdb Mongo.'.format(len(mongo_team_syncs)))
+
+        # create new dict of team syncs ready for firestore
+        team_syncs = {}
+        for sync in sorted(mongo_team_syncs, key=lambda x: x['google_group']):
+            del sync['_id']
+            email = sync['google_group']
+
+            print('Getting group and members: {}'.format(email))
+            # get group
+            group = g.directory().get_group(email)
+            # get members
+            members = []
+            for m in g.directory().get_members_recursively(email):
+                members.append(m['id'])
+
+            new_sync = {
+                # sync info
+                'created': sync['created'],
+                'modified': sync['modified'],
+                # group info
+                'group_email': group['email'],
+                'group_id': group['id'],
+                'group_members': members,
+                'group_name': group['name'],
+                # team info
+                'team_id': sync['github_team'],
+                'team_name': sync['github_team_name'],
+                'team_slug': sync['github_team_slug'],
+            }
+            team_id = str(sync['github_team'])
+            if team_id in team_syncs:
+                print('Duplicate Sync: {} -> {}'.format(email, team_id))
+            team_syncs[team_id] = new_sync
+
+        # get current team syncs from destination - broad-github-app firestore
+        projects = ['broad-github-app', 'broad-github-app-dev']
+        for project in projects:
+            print(g.firestore(project).update_collection_batch(
+                'team_syncs',
+                team_syncs,
+                delete_docs=True,
+            ))
+
+    #
+    # Update Users
+    #
+    def update_user(self, github_id, login, token, user):
+        """Update a single user."""
         # get last modified date from current record
         last_modified = None
-        if self.github_users.get(github_id):
-            current = self.github_users[github_id]
-            updated_at = dateparser.parse(current['updated_at'])
+        if user:
+            updated_at = dateparser.parse(user['updated_at'])
             last_modified = updated_at.strftime('%a, %d %b %Y %H:%M:%S GMT')
 
         # authenticate with user's token
@@ -393,7 +454,10 @@ class Update(object):
             return user
 
         # print(json.dumps(user, indent=2, sort_keys=True))
-        print('Updated user: {}'.format(user['login'].lower()))
+        if user:
+            print('Updated user: {}'.format(new['login'].lower()))
+        else:
+            print('Added user: {}'.format(new['login'].lower()))
 
         # check user_id
         user_id = str(new['id'])
@@ -419,13 +483,24 @@ class Update(object):
         github_users = []
         updates = []
         for email in sorted(self.users, key=lambda x: self.users[x]['login'].lower()):
-            user = self.users[email]
-            new = self.update_user(user)
+            # get user from github app with stored token
+            linked_user = self.users[email]
+            github_id = linked_user['github_id']
+            login = linked_user['login']
+            token = linked_user['token']
+
+            # get cached user from firestore
+            user = self.github_users.get(github_id, {})
+            new = self.update_user(github_id, login, token, user)
+
+            # add updated user to list
             github_users.append(new)
+
+            # if updated, add to updates
             if new != user:
                 updates.append(new)
 
         if self.verbose:
-            print('Updated {} github_users.'.format(len(updates)))
+            print('Updated {} GitHub Users.'.format(len(updates)))
 
         return github_users
